// vertex shader to fragment shader
varying vec3 vNormal;    // view space normal
varying vec3 wPos;       // world-space coordinate of the vertex
#ifdef ENABLE_TEXTURING
	varying vec2 vTexCoord;
#endif

// scene parameters
uniform vec4 ambient;

// the surface parameters of the mesh
uniform float Kd;
uniform float Ks;
uniform float Ka;
uniform float n_specular;

#ifdef ENABLE_TEXTURING
	uniform sampler2D texSource;
#endif

vec4 vDiffuse;
vec4 vSpecular;

void main(void) {   
	vec3 vNormal_N = normalize(vNormal);
	
	// Compute the light direction in view space:
	vec3 vViewLightDir = normalize(mat3(re_ViewMatrix) * re_SunLights_direction[0]);

	// Compute reflection vector in view space:
	vec3 vReflect = normalize( 2.0 * dot(vNormal_N, vViewLightDir) * vNormal_N - vViewLightDir );         

	// Compute view vector (view space):
	vec3 vView = -normalize(wPos);                

	// Compute diffuse and ambient contribution: 
	vDiffuse = ambient * Ka + re_SunLights_color[0] * Kd * max(0.0, dot(vNormal_N, vViewLightDir));

	// Compute specular term:
	vSpecular = re_SunLights_color[0] * Ks * pow(max(0.0, dot(vReflect, vView)), n_specular);

#ifdef ENABLE_TEXTURING
	gl_FragColor = texture2D(texSource, vTexCoord ) * vDiffuse;
#else	
	gl_FragColor = vDiffuse;
#endif
	gl_FragColor += vSpecular;
}
