/****************************************************************************
Copyright 2010 Bilkent University

Licensed under the Apache License, Version 2.0 (the "License"); 
you may not use this file except in compliance with the License. 
You may obtain a copy of the License at 
http://www.apache.org/licenses/LICENSE-2.0 

Unless required by applicable law or agreed to in writing, software 
distributed under the License is distributed on an "AS IS" BASIS, 
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
See the License for the specific language governing permissions and 
limitations under the License. 
****************************************************************************/
#ifndef _RENG_PLATFORM_H_
#define _RENG_PLATFORM_H_

#include "REng/Config.h"

/************************************************************************/
/* PLATFORM & COMPILER                                                  */
/************************************************************************/

/* Find the compiler type and version. */
#if defined( _MSC_VER )
#	define RENG_COMPILER RENG_COMPILER_MSVC
#	define RENG_COMP_VER _MSC_VER
#elif defined( __GNUC__ )
#	define RENG_COMP_VER (((__GNUC__)*100) + (__GNUC_MINOR__*10) + __GNUC_PATCHLEVEL__)
#	ifdef __OMAP__
#		define RENG_COMPILER RENG_COMPILER_ARM
#	else
#		define RENG_COMPILER RENG_COMPILER_GNUC
#	endif
#else
#	pragma error "Unsupported compiler."
#endif

/************************************************************************/
/* CHECK INCONSISTENCIES                                                */
/************************************************************************/

#if defined(__linux__)
#	define USING_XLIB
#else
#	define USING_WINCLS
#endif

/************************************************************************/
/* INCLUDE HEADERS BASED ON PLATFORM SETTINGS                           */
/************************************************************************/

#if RENG_COMPILER == RENG_COMPILER_MSVC
#	undef min
#	undef max
#	define NOMINMAX
#	define WIN32_LEAN_AND_MEAN
#endif

#if RENG_GL_PLATFORM == RENG_GL_PLATFORM_ES
#	define USING_EGL
#  define GLEW_USE_GL_ES
#else
#	if RENG_COMPILER == RENG_COMPILER_MSVC
#		define USING_WGL
#	endif
#	if RENG_COMPILER == RENG_COMPILER_GNUC
#		define USING_GLX
#	endif
#endif

// Include OpenGL header info
#define GLEW_NO_GLU
#define GLEW_STATIC
#include "GL/glew.h" // Note: Does not include Windows.h


#if RENG_PLATFORM == RENG_PLATFORM_OMAP
#	ifndef USING_EGL
#		pragma error "Can only use EGL on OMAP platform"
#	endif
#endif

// Identify debug builds for internal usage
#undef RENG_DEBUG_BUILD 
#ifdef _DEBUG
#	define RENG_DEBUG_BUILD 
#endif
#ifdef DEBUG
#	define RENG_DEBUG_BUILD 
#endif

/************************************************************************/
/* COMPILER SPECIFIC SETTINGS                                           */
/************************************************************************/

//
// EXPORTING/IMPORTING SYMBOLS WHEN USING SHARED LIBRARIES

// modify export directive when compiling in windows using shared library
#if RENG_PLATFORM == RENG_PLATFORM_WINDOWS
#	if defined( RENG_SHARED_LIB )
#		if defined( RENG_IMPORT_LIB )
#			define RENGAPI __declspec( dllimport )
#		else
#			define RENGAPI __declspec( dllexport )
#		endif
#	else
#		define RENGAPI
#	endif
#else
#	define RENGAPI
#endif


// Disable MSVC warnings, if your are compiling using MSVC
// From OGRE source
#if RENG_COMPILER == RENG_COMPILER_MSVC

	// Turn off warnings generated by long std templates
	// This warns about truncation to 255 characters in debug/browse info
	#pragma warning (disable : 4786)
	// Turn off warnings generated by long std templates
	// This warns about truncation to 255 characters in debug/browse info
	#pragma warning (disable : 4503)
	// disable: "conversion from 'double' to 'float', possible loss of data
	#pragma warning (disable : 4244)
	// disable: "truncation from 'double' to 'float'
	#pragma warning (disable : 4305)
	// disable: "non dll-interface class used as base for dll-interface class"
	// Happens when deriving from Singleton because bug in compiler ignores
	// template export
	#pragma warning (disable : 4275)
	// disable: "C++ Exception Specification ignored"
	// This is because MSVC 6 did not implement all the C++ exception
	// specifications in the ANSI C++ draft.
	#pragma warning( disable : 4290 )
	// disable: "no suitable definition provided for explicit template
	// instantiation request" Occurs in VC7 for no justifiable reason on all
	// #includes of Singleton
	#pragma warning( disable: 4661)
	// disable: deprecation warnings when using CRT calls in VC8
	// These show up on all C runtime lib code in VC8, disable since they clutter
	// the warnings with things we may not be able to do anything about (e.g.
	// generated code from nvparse etc). I doubt very much that these calls
	// will ever be actually removed from VC anyway, it would break too much code.
	#pragma warning( disable: 4996)
	// disable: "<type> needs to have dll-interface to be used by clients'
	#pragma warning (disable : 4251)


#endif // RENG_COMPILER == RENG_COMPILER_MSVC

#endif // _RENG_PLATFORM_H_

