
inline bool RigidBody::isSimulated() const {
	return dBodyIsEnabled(mID)!=0;
}

//////////////////////////////////////////////////////////////////////////
// GEOMS
//////////////////////////////////////////////////////////////////////////

inline dBodyID RigidBody::_getID() const {
	return mID;
}
inline size_t RigidBody::getGeomCount() const {
	size_t toRet(0);
	for(dGeomID gid = dBodyGetFirstGeom(mID); gid ; gid = dGeomGetBodyNext(gid)) ++toRet;
	return toRet;
}
inline Geom* RigidBody::getGeom(size_t index){
	dGeomID gid = dBodyGetFirstGeom(mID);
	for(size_t i=0 ; i<index ; ++i) gid = dGeomGetBodyNext(gid);
	return static_cast<Geom*>(dGeomGetData(gid));
}
inline const Geom* RigidBody::getGeom(size_t index) const{
	dGeomID gid = dBodyGetFirstGeom(mID);
	for(size_t i=0 ; i<index ; ++i) gid = dGeomGetBodyNext(gid);
	return static_cast<Geom*>(dGeomGetData(gid));
}

//////////////////////////////////////////////////////////////////////////
// FORCE / TORQUE
//////////////////////////////////////////////////////////////////////////

inline void RigidBody::addForce(const REng::Vector3& force){
	dBodyAddForce(mID, force[0], force[1], force[2]);
	// wake up body
	setSimulated(true);
}
inline void RigidBody::addRelForce(const REng::Vector3& force){
	dBodyAddRelForce(mID, force[0], force[1], force[2]);
	// wake up body
	setSimulated(true);
}
inline void RigidBody::addForceAtPos(const REng::Vector3& force, const REng::Vector3& pos){
	dBodyAddForceAtPos(mID, 
		force[0], force[1], force[2], 
		WORLD_TO_SIM(pos[0]), WORLD_TO_SIM(pos[1]), WORLD_TO_SIM(pos[2]));
	// wake up body
	setSimulated(true);
}
inline void RigidBody::addForceAtRelPos(const REng::Vector3& force, const REng::Vector3& pos){
	dBodyAddForceAtRelPos(mID, 
		force[0], force[1], force[2],
		WORLD_TO_SIM(pos[0]), WORLD_TO_SIM(pos[1]), WORLD_TO_SIM(pos[2]));
	// wake up body
	setSimulated(true);
}

inline void RigidBody::addTorque(const REng::Vector3& force){
	dBodyAddTorque(mID, force[0], force[1], force[2]);
	// wake up body
	setSimulated(true);
}
inline void RigidBody::addRelTorque(const REng::Vector3& force){
	dBodyAddRelTorque(mID, force[0], force[1], force[2]);
	// wake up body
	setSimulated(true);
}

inline void RigidBody::setTotalForce(const REng::Vector3& force){
	dBodySetForce(mID, force[0], force[1], force[2] );
	// wake up body
	setSimulated(true);
}
inline void RigidBody::setTotalTorque(const REng::Vector3& force){
	dBodySetTorque(mID, force[0], force[1], force[2] );
	// wake up body
	setSimulated(true);
}
inline REng::Vector3 RigidBody::getTotalForce() const{
	const dReal * tmp = dBodyGetForce (mID);
	return REng::Vector3(tmp[0], tmp[1], tmp[2]);
}
inline REng::Vector3 RigidBody::getTotalTorque() const{
	const dReal * tmp = dBodyGetTorque (mID);
	return REng::Vector3(tmp[0], tmp[1], tmp[2]);
}

//////////////////////////////////////////////////////////////////////////
// JOINTS
//////////////////////////////////////////////////////////////////////////

inline size_t RigidBody::getJointCount() const{
	return dBodyGetNumJoints(mID);
}
inline dJointID RigidBody::getJoint(size_t index){
	return dBodyGetJoint(mID,(int)index);
}

inline void RigidBody::setGravityApplied(bool flag){
	dBodySetGravityMode( mID, (flag?1:0) );
}
inline bool RigidBody::isGravityApplied() const{
	return dBodyGetGravityMode(mID)!=0;
}

inline void RigidBody::setKinematic(bool flag){
	if(flag==true){
		dBodySetKinematic(mID);
	} else {
		dBodySetDynamic(mID);
	}
}
inline bool RigidBody::isKinematic() const{
	return dBodyIsKinematic(mID)!=0;
}

inline void RigidBody::setGyroscobic(bool flag){
	dBodySetGyroscopicMode(mID,flag);
}
inline bool RigidBody::isGyroscobic() const{
	return dBodyGetGyroscopicMode(mID)!=0;
}
inline void RigidBody::setFiniteRotation(bool mode){
	dBodySetFiniteRotationMode(mID,mode);
}
inline void RigidBody::setFiniteRotationAxis(const REng::Vector3& axis){
	dBodySetFiniteRotationAxis(mID, axis[0], axis[1], axis[2]);
}
inline bool RigidBody::hasFiniteRotation() const{
	return dBodyGetFiniteRotationMode(mID)!=0;
}
inline REng::Vector3 RigidBody::getFiniteRotationAxis() const{
	dVector3 r;
	dBodyGetFiniteRotationAxis(mID,r);
	return REng::Vector3(r[0],r[1],r[2]);
}
