
inline World::World(){
	mID = dWorldCreate();
	damping.mID = this->mID;
	autoSleep.mID = this->mID;
	quickStep.mID = this->mID;
	mActiveSpace = 0;
	mStepMethod = WorldStep_Quick;
	mStepTime = 0.005f;
	mSimTimeAccum = 0;
	mListener = 0;
	mSimulationSpeed = 1.0f;
#ifdef _DEBUG
	mStepTime = 1.0f/60.0f;
#endif
}
inline World::World(const World& rhs){ (void) rhs; }
inline World& World::operator=(const World& rhs){ (void) rhs; return *this; }
inline dWorldID World::_getID(){
	return mID;
}

inline void World::attachListener(WorldSimListener* listener){
	mListener = listener;
}
inline WorldSimListener* World::getListener(){
	return mListener;
}
inline void World::detachListener(){
	mListener = 0;
}
inline bool World::isListenerActive() const{
	return mListener!=0;
}

inline RigidBody* World::createRigidBody(){
	return new RigidBody(*this);
}
inline RigidBodyBox* World::createRigidBodyBox(Real density, const REng::Vector3& size){
	return new RigidBodyBox(*this,density,size);
}
inline RigidBodySphere* World::createRigidBodySphere(Real density, Real radius){
	return new RigidBodySphere(*this,density,radius);
}

inline void World::setGravityForce(const REng::Vector3& g){
	mGravityCache = g;
	dWorldSetGravity(mID,
		WORLD_TO_SIM(g[0]),
		WORLD_TO_SIM(g[1]),
		WORLD_TO_SIM(g[2]));
}
inline void World::disableGravity(){
	dWorldSetGravity(mID,0.,0.,0.);
}
inline void World::enableGravity(){
	setGravityForce(mGravityCache);
}
inline void World::toggleGravity(){
	REng::Vector3 g(getGravityForce());
	if(g==REng::Vector3(0.,0.,0.))
		enableGravity();
	else
		disableGravity();
}
inline bool World::isGravityEnabled() const{
	return getGravityForce()!=REng::Vector3(0.,0.,0.);
}
inline REng::Vector3 World::getGravityForce() const{
	dVector3 g;
	dWorldGetGravity(mID,g);
	return REng::Vector3(
		SIM_TO_WORLD(g[0]),
		SIM_TO_WORLD(g[1]),
		SIM_TO_WORLD(g[2]));
}
inline void World::setERP(Real erp){
	dWorldSetERP(mID,erp);
}
inline Real World::getERP() const{
	return dWorldGetERP(mID);
}
inline void World::setCFM(Real cfm){
	dWorldSetCFM(mID,cfm);
}
inline Real World::getCFM() const{
	return dWorldGetCFM(mID);
}
inline void World::setStepTime(float sec){
	mStepTime = sec;
}
inline float World::getStepTime(){
	return mStepTime;
}
inline void World::step(){
	if(mStepMethod==WorldStep_Normal){
		dWorldStep(mID,mStepTime);
	}
	if(mStepMethod==WorldStep_Quick){
		dWorldQuickStep(mID,mStepTime);
	}
}
inline bool World::canStep(){
	return (mSimTimeAccum>=mStepTime);
}

inline QuickStepParams::QuickStepParams() : mID(0) { ; }
inline void QuickStepParams::setIterationCount(size_t i){
	dWorldSetQuickStepNumIterations(mID,int(i));
}
inline size_t QuickStepParams::getIterationCOunt() const{
	return (size_t)dWorldGetQuickStepNumIterations(mID);
}
inline void QuickStepParams::setOverRelaxation(Real over_relax){
	dWorldSetQuickStepW(mID,over_relax);
}
inline Real QuickStepParams::getOverRelaxation() const{
	return dWorldGetQuickStepW(mID);
}

inline void World::setContactMaxCorrectingVelocity(Real vel){
	dWorldSetContactMaxCorrectingVel(mID,vel);
}
inline Real World::getMaxCorrectingVelocity() const{
	return dWorldGetContactMaxCorrectingVel(mID);
}
inline void World::setContactSurfaceLayerDepth(Real depth){
	dWorldSetContactSurfaceLayer(mID,depth);
}
inline Real World::getContactSurfaceLayerDepth() const{
	return dWorldGetContactSurfaceLayer(mID);
}

inline WorldSimListener::~WorldSimListener(){ }

inline void WorldSimListener::preStep(){ }
inline void WorldSimListener::postStep(){ }


